from __future__ import annotations

from collections.abc import Collection
from typing import Union, Dict, Any, List, cast

from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.common.graph.graph_builder.utils import calculate_hash, join_trimmed_strings
from checkov.common.graph.graph_builder.variable_rendering.breadcrumb_metadata import BreadcrumbMetadata
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.terraform.graph_builder.graph_components.block_types import BlockType


class Block:
    __slots__ = (
        "attributes",
        "block_type",
        "breadcrumbs",
        "changed_attributes",
        "config",
        "id",
        "name",
        "path",
        "source",
        "has_dynamic_block",
        "dynamic_attributes",
        "foreach_attrs"
    )

    def __init__(
            self,
            name: str,
            config: Dict[str, Any],
            path: str,
            block_type: str,
            attributes: Dict[str, Any],
            id: str = "",
            source: str = "",
            has_dynamic_block: bool = False,
            dynamic_attributes: dict[str, Any] | None = None
    ) -> None:
        """
            :param name: unique name given to the block, for example
            :param config: the section in tf_definitions that belong to this block
            :param path: the file location of the block
            :param block_type: str
            :param attributes: dictionary of the block's original attributes in the origin file
        """
        self.name = name
        self.config = pickle_deepcopy(config)
        self.path = path
        self.block_type = block_type
        self.attributes = attributes
        self.id = id
        self.source = source
        self.changed_attributes: Dict[str, List[Any]] = {}
        self.breadcrumbs: Dict[str, List[Dict[str, Any]]] = {}

        attributes_to_add = self._extract_inner_attributes(has_dynamic_block, dynamic_attributes)
        self.attributes.update(attributes_to_add)

    def _extract_inner_attributes(self, has_dynamic_block: bool = False, dynamic_attributes: dict[str, Any] | None = None) -> Dict[str, Any]:
        attributes_to_add = {}
        for attribute_key, attribute_value in self.attributes.items():
            if has_dynamic_block and attribute_key in dynamic_attributes.keys():  # type: ignore
                continue
            if isinstance(attribute_value, dict) or (
                isinstance(attribute_value, list) and len(attribute_value) > 0 and isinstance(attribute_value[0], dict)
            ):
                inner_attributes = self.get_inner_attributes(
                    attribute_key=attribute_key,
                    attribute_value=attribute_value,
                )
                attributes_to_add.update(inner_attributes)
        return attributes_to_add

    def __str__(self) -> str:
        return f"{self.block_type}: {self.name}"

    def get_attribute_dict(self, add_hash: bool = True) -> Dict[str, Any]:
        """
           :return: map of all the block's native attributes (from the source file),
           combined with the attributes generated by the module builder.
           If the attributes are not a primitive type, they are converted to strings.
           """
        base_attributes = self.get_base_attributes()
        self.get_origin_attributes(base_attributes)

        if hasattr(self, "module_dependency") and hasattr(self, "module_dependency_num"):
            base_attributes[CustomAttributes.MODULE_DEPENDENCY] = self.module_dependency
            base_attributes[CustomAttributes.MODULE_DEPENDENCY_NUM] = self.module_dependency_num

        if self.changed_attributes:
            # add changed attributes only for calculating the hash
            base_attributes["changed_attributes"] = sorted(self.changed_attributes.keys())

        if self.breadcrumbs:
            sorted_breadcrumbs = dict(sorted(self.breadcrumbs.items()))
            base_attributes[CustomAttributes.RENDERING_BREADCRUMBS] = sorted_breadcrumbs

        if hasattr(self, 'foreach_attrs'):
            base_attributes[CustomAttributes.FOREACH_ATTRS] = self.foreach_attrs

        if hasattr(self, 'source_module_object'):
            base_attributes[CustomAttributes.SOURCE_MODULE_OBJECT] = self.source_module_object

        if add_hash:
            base_attributes[CustomAttributes.HASH] = calculate_hash(base_attributes)

        if self.block_type == BlockType.DATA:
            base_attributes[CustomAttributes.RESOURCE_TYPE] = f'data.{self.id.split(".")[0]}'

        if self.block_type == BlockType.MODULE:
            # since module names are user defined we are just setting 'module' as resource type for easier searching
            base_attributes[CustomAttributes.RESOURCE_TYPE] = "module"

        if self.block_type == BlockType.PROVIDER:
            # provider_name is always a string, base_attributes needs better typing pipenv run mypy
            provider_name = cast(str, base_attributes[CustomAttributes.BLOCK_NAME])
            provider_type = provider_name.split(".")[0]
            # ex: provider.aws
            base_attributes[CustomAttributes.RESOURCE_TYPE] = f"provider.{provider_type}"

        if "changed_attributes" in base_attributes:
            # removed changed attributes if it was added previously for calculating hash.
            del base_attributes["changed_attributes"]

        return base_attributes

    def get_origin_attributes(self, base_attributes: Dict[str, Any]) -> None:
        for attribute_key in list(self.attributes.keys()):
            attribute_value = self.attributes[attribute_key]
            if isinstance(attribute_value, list) and len(attribute_value) == 1:
                if '.' not in attribute_key:
                    attribute_value = attribute_value[0]
            # needs to be checked before adding anything to 'base_attributes'
            if attribute_key == "self":
                base_attributes["self_"] = attribute_value
                continue
            if isinstance(attribute_value, (list, dict)):
                inner_attributes = self.get_inner_attributes(attribute_key, attribute_value, False)
                base_attributes.update(inner_attributes)

            base_attributes[attribute_key] = attribute_value

    def get_hash(self) -> str:
        attributes_dict = self.get_attribute_dict()
        return attributes_dict.get(CustomAttributes.HASH, "")

    def update_attribute(
        self,
        attribute_key: str,
        attribute_value: Any,
        change_origin_id: int | None,
        previous_breadcrumbs: list[BreadcrumbMetadata],
        attribute_at_dest: str | None,
        transform_step: bool = False,
    ) -> None:
        self.update_inner_attribute(
            attribute_key=attribute_key,
            nested_attributes=self.attributes,
            value_to_update=attribute_value
        )

        if (
            self._should_add_previous_breadcrumbs(change_origin_id, previous_breadcrumbs, attribute_at_dest)
            and change_origin_id is not None
        ):
            previous_breadcrumbs.append(BreadcrumbMetadata(change_origin_id, attribute_at_dest))

        # update the numbered attributes, if the new value is a list
        if attribute_value and isinstance(attribute_value, list):
            self.update_list_attribute(attribute_key=attribute_key, attribute_value=attribute_value)

        attribute_key_parts = attribute_key.split(".")
        if len(attribute_key_parts) == 1:
            self.attributes[attribute_key] = attribute_value
            if self._should_set_changed_attributes(change_origin_id, attribute_at_dest):
                self.changed_attributes[attribute_key] = previous_breadcrumbs
            return
        for i in range(len(attribute_key_parts)):
            key = join_trimmed_strings(char_to_join=".", str_lst=attribute_key_parts, num_to_trim=i)
            if key.find(".") > -1:
                additional_changed_attributes = self.extract_additional_changed_attributes(key)
                self.attributes[key] = attribute_value
                end_key_part = attribute_key_parts[len(attribute_key_parts) - 1 - i]
                if transform_step and end_key_part in ("1", "2"):
                    # if condition logic during the transform step breaks the values
                    return
                attribute_value = {end_key_part: attribute_value}
                if self._should_set_changed_attributes(change_origin_id, attribute_at_dest):
                    self.changed_attributes[key] = previous_breadcrumbs
                    if additional_changed_attributes:
                        for changed_attribute in additional_changed_attributes:
                            self.changed_attributes[changed_attribute] = previous_breadcrumbs

    def update_inner_attribute(
        self, attribute_key: str, nested_attributes: list[Any] | dict[str, Any], value_to_update: Any
    ) -> None:
        split_key = attribute_key.split(".")
        i = 1
        curr_key = ".".join(split_key[0:i])
        if isinstance(nested_attributes, list):
            if curr_key.isnumeric():
                curr_key_int = int(curr_key)
                if curr_key_int < len(nested_attributes):
                    if not isinstance(nested_attributes[curr_key_int], dict):
                        nested_attributes[curr_key_int] = value_to_update
                    else:
                        self.update_inner_attribute(
                            ".".join(split_key[i:]), nested_attributes[curr_key_int], value_to_update
                        )
            else:
                for inner in nested_attributes:
                    self.update_inner_attribute(curr_key, inner, value_to_update)
        elif isinstance(nested_attributes, dict):
            while curr_key not in nested_attributes and i <= len(split_key):
                i += 1
                curr_key = ".".join(split_key[0:i])
            if attribute_key in nested_attributes.keys():
                nested_attributes[attribute_key] = value_to_update
            if len(split_key) == 1 and len(curr_key) > 0:
                nested_attributes[curr_key] = value_to_update
            elif curr_key in nested_attributes.keys():
                self.update_inner_attribute(".".join(split_key[i:]), nested_attributes[curr_key], value_to_update)

    def update_list_attribute(self, attribute_key: str, attribute_value: Any) -> None:
        """Updates list attributes with their index"""

        for idx, value in enumerate(attribute_value):
            self.attributes[f"{attribute_key}.{idx}"] = value

    @staticmethod
    def _should_add_previous_breadcrumbs(
        change_origin_id: int | None, previous_breadcrumbs: list[BreadcrumbMetadata], attribute_at_dest: str | None
    ) -> bool:
        return not previous_breadcrumbs or previous_breadcrumbs[-1].vertex_id != change_origin_id

    def extract_additional_changed_attributes(self, attribute_key: str) -> List[str]:
        """
        override in case of a special case where additional attributes are needed to be tracked included in self.changed_attributes
        and self.breadcrumbs, such as terraform dynamic blocks
        :param attribute_key: JSONPath notation of an attribute key that is used for extraction
        :return: list of the additional attributes, in JSONPath notation
        """
        return []

    @staticmethod
    def _should_set_changed_attributes(change_origin_id: int | None, attribute_at_dest: str | None) -> bool:
        return True

    def get_export_data(self) -> Dict[str, Union[bool, str]]:
        return {"type": self.block_type, "name": self.name, "path": self.path}

    def get_base_attributes(self) -> Dict[str, Union[str, List[str], Dict[str, Any]]]:
        return {
            CustomAttributes.BLOCK_NAME: self.name,
            CustomAttributes.BLOCK_TYPE: self.block_type,
            CustomAttributes.FILE_PATH: self.path,
            CustomAttributes.CONFIG: self.config,
            CustomAttributes.LABEL: str(self),
            CustomAttributes.ID: self.id,
            CustomAttributes.SOURCE: self.source,
        }

    @classmethod
    def get_inner_attributes(
        cls,
        attribute_key: str,
        attribute_value: str | List[str] | dict[str, Any],
        strip_list: bool = True  # used by subclass
    ) -> dict[str, Any]:
        inner_attributes: dict[str, Any] = {}

        if isinstance(attribute_value, (dict, list)):
            inner_attributes[attribute_key] = [None] * len(attribute_value) if isinstance(attribute_value, list) else {}
            iterator: Collection[int] | Collection[str] = range(len(attribute_value)) if isinstance(
                attribute_value, list
            ) else list(
                attribute_value.keys()
            )
            for key in iterator:
                if key != "":
                    inner_key = f"{attribute_key}.{key}"
                    inner_value = attribute_value[key]  # type:ignore[index]
                    inner_attributes.update(cls.get_inner_attributes(inner_key, inner_value))
                    inner_attributes[attribute_key][key] = inner_attributes[inner_key]
                else:
                    del attribute_value[key]  # type:ignore[arg-type]
        else:
            inner_attributes[attribute_key] = attribute_value
        return inner_attributes



from __future__ import annotations

import json
import logging
from collections.abc import Iterable
from typing import Any, TYPE_CHECKING

import yaml
from termcolor import colored

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.util.env_vars_config import env_vars_config

if TYPE_CHECKING:
    from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver

logger = logging.getLogger(__name__)


def graph_check(check_id: str, check_name: str) -> None:
    if not env_vars_config.EXPERIMENTAL_GRAPH_DEBUG:
        return

    print(f'\nEvaluating graph policy: "{check_id}" - "{check_name}"')


def resource_types(resource_types: Iterable[str], resource_count: int, operator: str) -> None:
    if not env_vars_config.EXPERIMENTAL_GRAPH_DEBUG:
        return

    resource_types_str = '", "'.join(resource_types)
    print(
        f'\nFound {resource_count} resources with resource types: "{resource_types_str}" to check against operator: "{operator}"'
    )


def attribute_block(
    resource_types: Iterable[str],
    attribute: str | None,
    operator: str,
    value: str | list[str] | None,
    resource: dict[str, Any],
    status: str,
) -> None:
    if not env_vars_config.EXPERIMENTAL_GRAPH_DEBUG:
        return

    attribute_block_conf = _create_attribute_block(
        resource_types=resource_types, attribute=attribute, operator=operator, value=value
    )
    color = "green" if status == "passed" else "red"

    print("\nEvaluated block:\n")
    print(colored(yaml.dump([attribute_block_conf], sort_keys=False), "blue"))
    print("and got:")
    print(colored(f'\nResource "{resource[CustomAttributes.ID]}" {status}:', color))
    print(colored(json.dumps(resource[CustomAttributes.CONFIG], indent=2), "yellow"))


def connection_block(
    resource_types: Iterable[str],
    connected_resource_types: Iterable[str],
    operator: str,
    passed_resources: list[dict[str, Any]],
    failed_resources: list[dict[str, Any]],
) -> None:
    if not env_vars_config.EXPERIMENTAL_GRAPH_DEBUG:
        return

    connection_block_conf = _create_connection_block(
        resource_types=resource_types,
        connected_resource_types=connected_resource_types,
        operator=operator,
    )

    passed_resources_str = '", "'.join(resource[CustomAttributes.ID] for resource in passed_resources)
    failed_resources_str = '", "'.join(resource[CustomAttributes.ID] for resource in failed_resources)

    print("\nEvaluated blocks:\n")
    print(colored(yaml.dump([connection_block_conf], sort_keys=False), "blue"))
    print("and got:\n")
    print(colored(f'Passed resources: "{passed_resources_str}"', "green"))
    print(colored(f'Failed resources: "{failed_resources_str}"', "red"))


def complex_connection_block(
    solvers: list[BaseSolver],
    operator: str,
    passed_resources: list[dict[str, Any]],
    failed_resources: list[dict[str, Any]],
) -> None:
    if not env_vars_config.EXPERIMENTAL_GRAPH_DEBUG:
        return

    # to prevent circular dependencies
    from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
    from checkov.common.checks_infra.solvers.complex_solvers.base_complex_solver import BaseComplexSolver
    from checkov.common.checks_infra.solvers.connections_solvers.base_connection_solver import BaseConnectionSolver
    from checkov.common.checks_infra.solvers.connections_solvers.complex_connection_solver import (
        ComplexConnectionSolver,
    )
    from checkov.common.checks_infra.solvers.filter_solvers.base_filter_solver import BaseFilterSolver

    complex_connection_block = []

    for solver in solvers:
        if isinstance(solver, BaseAttributeSolver):
            block = _create_attribute_block(
                resource_types=solver.resource_types,
                attribute=solver.attribute,
                operator=solver.operator,
                value=solver.value,
            )
        elif isinstance(solver, BaseFilterSolver):
            block = _create_filter_block(attribute=solver.attribute, operator=solver.operator, value=solver.value)
        elif isinstance(solver, (ComplexConnectionSolver, BaseComplexSolver)):
            # ComplexConnectionSolver check needs to be before BaseConnectionSolver, because it is a subclass
            block = {solver.operator: ["..." for _ in solver.solvers]}
        elif isinstance(solver, BaseConnectionSolver):
            block = _create_connection_block(
                resource_types=solver.resource_types,
                connected_resource_types=solver.connected_resources_types,
                operator=solver.operator,
            )
        else:
            logger.info(f"Unsupported solver type {type(solver)} found")
            continue

        complex_connection_block.append(block)

    passed_resources_str = '", "'.join(resource[CustomAttributes.ID] for resource in passed_resources)
    failed_resources_str = '", "'.join(resource[CustomAttributes.ID] for resource in failed_resources)

    print("\nEvaluated blocks:\n")
    print(colored(yaml.dump([{operator: complex_connection_block}], sort_keys=False), "blue"))
    print("and got:\n")
    print(colored(f'Passed resources: "{passed_resources_str}"', "green"))
    print(colored(f'Failed resources: "{failed_resources_str}"', "red"))


def _create_attribute_block(
    resource_types: Iterable[str], attribute: str | None, operator: str, value: str | list[str] | None
) -> dict[str, Any]:
    attribute_block_conf = {
        "cond_type": "attribute",
        "resource_types": resource_types,
        "attribute": attribute,
        "operator": operator,
    }
    if value is not None:
        attribute_block_conf["value"] = value

    return attribute_block_conf


def _create_connection_block(
    resource_types: Iterable[str], connected_resource_types: Iterable[str], operator: str
) -> dict[str, Any]:
    attribute_block_conf = {
        "cond_type": "connection",
        "resource_types": resource_types,
        "connected_resource_types": connected_resource_types,
        "operator": operator,
    }
    return attribute_block_conf


def _create_filter_block(attribute: str | None, operator: str, value: str | list[str]) -> dict[str, Any]:
    attribute_block_conf = {
        "cond_type": "filter",
        "attribute": attribute,
        "operator": operator,
        "value": value,
    }
    return attribute_block_conf



from __future__ import annotations

import re
from typing import TYPE_CHECKING

from checkov.secrets.parsers.multiline_parser import BaseMultiLineParser

if TYPE_CHECKING:
    from detect_secrets.util.code_snippet import CodeSnippet

START_OBJ = re.compile(r"^\s*\w*\s*=?\s*{\s*$")
END_OBJ = re.compile(r"^\s*}\s*$")
COMMENT_PREFIX = re.compile(r"^[\s]*(#|//)")


class TerraformMultiLineParser(BaseMultiLineParser):
    def consecutive_lines_in_same_object(
        self,
        raw_context: CodeSnippet | None,
        other_line_idx: int,
    ) -> bool:
        return bool(raw_context and 0 <= other_line_idx < len(raw_context.lines))

    @staticmethod
    def is_object_start(line: str) -> bool:
        return bool(re.match(START_OBJ, line))

    @staticmethod
    def is_object_end(line: str) -> bool:
        return bool(re.match(END_OBJ, line))

    @staticmethod
    def is_line_comment(line: str) -> bool:
        return bool(re.match(COMMENT_PREFIX, line))


terraform_multiline_parser = TerraformMultiLineParser()



from __future__ import annotations

import logging
import os
import platform

from abc import abstractmethod
from collections.abc import Iterable
from pathlib import Path
from typing import Any, TYPE_CHECKING, Callable
from typing_extensions import TypedDict

from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.models.enums import CheckResult
from checkov.common.typing import LibraryGraphConnector
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.output.github_actions_record import GithubActionsRecord
from checkov.common.output.record import Record
from checkov.common.output.report import Report, CheckType
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths, CHECKOV_CREATE_GRAPH
from checkov.common.runners.graph_manager import ObjectGraphManager
from checkov.common.typing import _CheckResult
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.runner_filter import RunnerFilter
from checkov.common.util.suppression import collect_suppressions_for_context

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
    from checkov.common.runners.graph_builder.local_graph import ObjectLocalGraph


class GhaMetadata(TypedDict):
    triggers: set[str]
    workflow_name: str
    jobs: dict[int, str]


class Runner(BaseRunner[ObjectGraphManager]):  # if a graph is added, Any needs to replaced
    def __init__(
        self,
        db_connector: LibraryGraphConnector | None = None,
        source: str | None = None,
        graph_class: type[ObjectLocalGraph] | None = None,
        graph_manager: ObjectGraphManager | None = None,
    ) -> None:
        super().__init__()
        self.definitions: dict[str, dict[str, Any] | list[dict[str, Any]]] = {}
        self.definitions_raw: dict[str, list[tuple[int, str]]] = {}
        self.map_file_path_to_gha_metadata_dict: dict[str, GhaMetadata] = {}
        self.root_folder: str | None = None

        if source and graph_class:
            # if they are not all set, then ignore it
            db_connector = db_connector or self.db_connector
            self.source = source
            self.graph_class = graph_class
            self.graph_manager = (
                graph_manager if graph_manager else ObjectGraphManager(source=self.source, db_connector=db_connector)
            )
            self.graph_registry = get_graph_checks_registry(self.check_type)

    def _load_files(
            self,
            files_to_load: list[str],
            filename_fn: Callable[[str], str] | None = None,
    ) -> None:
        files_to_load = [filename_fn(file) if filename_fn else file for file in files_to_load]
        results = parallel_runner.run_function(lambda f: (f, self._parse_file(f)), files_to_load)
        for file_result_pair in results:
            if file_result_pair is None:
                # this only happens, when an uncaught exception occurs
                continue

            file, result = file_result_pair
            if result:
                (self.definitions[file], self.definitions_raw[file]) = result
                definition = result[0]
                if self.check_type == CheckType.GITHUB_ACTIONS and isinstance(definition, dict):
                    workflow_name = definition.get('name', '')
                    triggers = self._get_triggers(definition)
                    jobs = self._get_jobs(definition)
                    self.map_file_path_to_gha_metadata_dict[file] = \
                        {"triggers": triggers, "workflow_name": workflow_name, "jobs": jobs}

    @abstractmethod
    def _parse_file(
            self, f: str
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        raise Exception("parser should be imported by deriving class")

    def run(
            self,
            root_folder: str | None = None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        registry = self.import_registry()

        report = Report(self.check_type)

        if not files and not root_folder:
            logging.debug("No resources to scan.")
            return report

        if not external_checks_dir and self.require_external_checks():
            logging.debug("The runner requires that external checks are defined.")
            return report
        if external_checks_dir:
            for directory in external_checks_dir:
                registry.load_external_checks(directory)

                if CHECKOV_CREATE_GRAPH and self.graph_registry:
                    self.graph_registry.load_external_checks(directory)

        if not self.context or not self.definitions:
            if files:
                self._load_files(files)

            if root_folder:
                self.root_folder = root_folder

                for root, d_names, f_names in os.walk(root_folder):
                    filter_ignored_paths(root, d_names, runner_filter.excluded_paths, self.included_paths())
                    filter_ignored_paths(root, f_names, runner_filter.excluded_paths, self.included_paths())
                    files_to_load = [os.path.join(root, f_name) for f_name in f_names]
                    self._load_files(files_to_load=files_to_load)

            self.context = self.build_definitions_context(definitions=self.definitions, definitions_raw=self.definitions_raw)

            if CHECKOV_CREATE_GRAPH and self.graph_registry and self.graph_manager:
                logging.info(f"Creating {self.source} graph")
                local_graph = self.graph_manager.build_graph_from_definitions(
                    definitions=self.definitions, graph_class=self.graph_class  # type:ignore[arg-type]  # the paths are just `str`
                )

                logging.info(f"Successfully created {self.source} graph")

                self.graph_manager.save_graph(local_graph)
        else:
            logging.info("Going to use existing graph")
            self.populate_metadata_dict()

        self.pbar.initiate(len(self.definitions))

        # run Python checks
        self.add_python_check_results(report=report, registry=registry, runner_filter=runner_filter, root_folder=root_folder)

        # run graph checks
        if CHECKOV_CREATE_GRAPH and self.graph_registry:
            self.add_graph_check_results(report=report, runner_filter=runner_filter)

        return report

    def add_python_check_results(
        self, report: Report, registry: BaseCheckRegistry, runner_filter: RunnerFilter, root_folder: str | Path | None
    ) -> None:
        """Adds Python check results to given report"""

        for file_path in self.definitions.keys():
            self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(file_path, root_folder)})
            skipped_checks = collect_suppressions_for_context(self.definitions_raw[file_path])

            if registry.report_type == CheckType.GITLAB_CI:
                registry.definitions_raw = self.definitions_raw[file_path]
            results = registry.scan(file_path, self.definitions[file_path], skipped_checks, runner_filter)  # type:ignore[arg-type] # this is overridden in the subclass
            for key, result in results.items():
                result_config = result["results_configuration"]
                start = 0
                end = 0
                check = result.pop("check", None)  # use pop to remove Check class which is not serializable from
                if check is None:
                    continue

                # result record
                if result_config:
                    end, start = self.get_start_end_lines(end, result_config, start)
                    if start == -1 and end == -1:
                        logging.info(f"Skipping line in file path {file_path} in key {key}")
                        continue
                if platform.system() == "Windows":
                    root_folder = os.path.split(file_path)[0]

                if self.check_type == CheckType.GITHUB_ACTIONS:
                    record: "Record" = GithubActionsRecord(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=result,
                        code_block=self.definitions_raw[file_path][start - 1:end + 1],
                        file_path=f"/{os.path.relpath(file_path, root_folder)}",
                        file_line_range=[start, end + 1],
                        resource=self.get_resource(file_path, key, check.supported_entities, start, end),  # type:ignore[arg-type]  # key is str not BaseCheck
                        evaluations=None,
                        check_class=check.__class__.__module__,
                        file_abs_path=os.path.abspath(file_path),
                        entity_tags=None,
                        severity=check.severity,
                        job=self.map_file_path_to_gha_metadata_dict[file_path]["jobs"].get(end, ''),
                        triggers=self.map_file_path_to_gha_metadata_dict[file_path]["triggers"],
                        workflow_name=self.map_file_path_to_gha_metadata_dict[file_path]["workflow_name"]
                    )
                else:
                    record = Record(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=result,
                        code_block=self.definitions_raw[file_path][start - 1:end + 1],
                        file_path=f"/{os.path.relpath(file_path, root_folder)}",
                        file_line_range=[start, end + 1],
                        resource=self.get_resource(file_path, key, check.supported_entities, start, end),  # type:ignore[arg-type]  # key is str not BaseCheck
                        evaluations=None,
                        check_class=check.__class__.__module__,
                        file_abs_path=os.path.abspath(file_path),
                        entity_tags=None,
                        severity=check.severity,
                    )
                report.add_record(record)
            self.pbar.update()
        self.pbar.close()

    def add_graph_check_results(self, report: Report, runner_filter: RunnerFilter) -> None:
        """Adds graph check results to given report"""

        root_folder = self.root_folder
        graph_checks_results = self.run_graph_checks_results(runner_filter, self.check_type)

        for check, check_results in graph_checks_results.items():
            for check_result in check_results:
                entity = check_result["entity"]
                entity_file_path: str = entity[CustomAttributes.FILE_PATH]

                if platform.system() == "Windows":
                    root_folder = os.path.split(entity_file_path)[0]

                clean_check_result: _CheckResult = {
                    "result": check_result["result"],
                    "evaluated_keys": check_result["evaluated_keys"],
                }

                start_line = entity[START_LINE]
                end_line = entity[END_LINE]
                code_block = self.get_code_block(entity=entity)

                self.add_inline_suppression(check=check, entity=entity, check_result=clean_check_result)

                if self.check_type == CheckType.GITHUB_ACTIONS:
                    if entity.get(CustomAttributes.BLOCK_NAME) == 'permissions' and start_line == 0 and end_line == 0:
                        # reconstruct permissions start-end lines since we do not have that information during graph build
                        for line in self.definitions_raw[entity_file_path]:
                            if line and 'permissions' in line[1]:
                                start_line = line[0]
                                end_line = line[0]
                                break

                    entity[CustomAttributes.ID] = self.get_resource(entity_file_path, entity[CustomAttributes.ID],
                                                                    entity[CustomAttributes.RESOURCE_TYPE],
                                                                    start_line, end_line, graph_resource=True)
                    record: "Record" = GithubActionsRecord(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=clean_check_result,
                        code_block=code_block,
                        file_path=f"/{os.path.relpath(entity_file_path, root_folder)}",
                        file_line_range=[start_line, end_line + 1],
                        resource=entity[CustomAttributes.ID],
                        evaluations=None,
                        check_class=check.__class__.__module__,
                        file_abs_path=os.path.abspath(entity_file_path),
                        entity_tags=None,
                        severity=check.severity,
                        job=self.map_file_path_to_gha_metadata_dict[entity_file_path]["jobs"].get(end_line, ''),
                        triggers=self.map_file_path_to_gha_metadata_dict[entity_file_path]["triggers"],
                        workflow_name=self.map_file_path_to_gha_metadata_dict[entity_file_path]["workflow_name"]
                    )
                else:
                    record = Record(
                        check_id=check.id,
                        bc_check_id=check.bc_id,
                        check_name=check.name,
                        check_result=clean_check_result,
                        code_block=code_block,
                        file_path=f"/{os.path.relpath(entity_file_path, root_folder)}",
                        file_line_range=[start_line, end_line + 1],
                        resource=entity[CustomAttributes.ID],
                        evaluations=None,
                        check_class=check.__class__.__module__,
                        file_abs_path=os.path.abspath(entity_file_path),
                        entity_tags=None,
                        severity=check.severity,
                    )

                record.set_guideline(check.guideline)
                report.add_record(record=record)

    def get_resource(self, file_path: str, key: str, supported_entities: Iterable[str],
                     start_line: int = -1, end_line: int = -1, graph_resource: bool = False) -> str:
        return f"{file_path}.{key}"

    @abstractmethod
    def get_start_end_lines(self, end: int, result_config: dict[str, Any], start: int) -> tuple[int, int]:
        raise Exception("should be handled by derived class")

    @abstractmethod
    def import_registry(self) -> BaseCheckRegistry:
        raise Exception("registry should be imported by deriving class")

    def require_external_checks(self) -> bool:
        return True

    @staticmethod
    def _change_files_path_to_relative(report: Report) -> None:
        for record in report.get_all_records():
            record.file_path = record.file_path.replace(os.getcwd(), "")
            record.resource = record.resource.replace(os.getcwd(), "")

    def build_definitions_context(
        self,
        definitions: dict[str, dict[str, Any] | list[dict[str, Any]]],
        definitions_raw: dict[str, list[tuple[int, str]]],
    ) -> dict[str, dict[str, Any]]:
        # if needed, should be overridden in the actual runner class
        return {}

    def get_code_block(self, entity: dict[str, Any]) -> list[tuple[int, str]]:
        """Returns the code block either from context or definitions_raw"""

        code_block: list[tuple[int, str]] = []

        entity_file_path = entity[CustomAttributes.FILE_PATH]

        if self.context:
            # not all runners have the 'context' attribute populated
            entity_id = entity[CustomAttributes.ID]
            entity_context = self.context[entity_file_path].get(entity_id)

            if entity_context:
                code_block = entity_context.get("code_lines")
            else:
                logging.info(f"Could not find context for resource {entity_id} in file {entity_file_path}")

        if not code_block:
            # fallback, if context extraction failed
            start_line = entity[START_LINE]
            end_line = entity[END_LINE]
            code_block = self.definitions_raw[entity_file_path][start_line - 1:end_line + 1]

        return code_block

    def add_inline_suppression(self, check: BaseGraphCheck, entity: dict[str, Any], check_result: _CheckResult) -> None:
        """Adjusts check result, if inline suppressed"""

        if self.context:
            # not all runners have the 'context' attribute populated
            entity_file_path = entity[CustomAttributes.FILE_PATH]
            entity_id = entity[CustomAttributes.ID]
            entity_context = self.context[entity_file_path].get(entity_id)

            if entity_context:
                skipped_check = next(
                    (
                        skipped_check
                        for skipped_check in entity_context.get("skipped_checks", [])
                        if skipped_check["id"] in (check.id, check.bc_id)
                    ),
                    None,
                )
                if skipped_check:
                    check_result["result"] = CheckResult.SKIPPED
                    check_result["suppress_comment"] = skipped_check.get("suppress_comment", "")

    def _get_triggers(self, definition: dict[str, Any]) -> set[str]:
        triggers_set = set()
        triggers = definition.get("on")
        try:
            if isinstance(triggers, str):
                triggers_set.add(triggers)
            elif isinstance(triggers, dict):
                triggers_set = {key for key in triggers.keys() if key != START_LINE and key != END_LINE}

        except Exception as e:
            logging.info(f"failed to parse workflow triggers due to:{str(e)}")
        return triggers_set

    def _get_jobs(self, definition: dict[str, Any]) -> dict[int, str]:
        end_line_to_job_name_dict: dict[int, str] = {}
        jobs = definition.get('jobs')
        if jobs:
            for job_name, job_instance in jobs.items():
                if not isinstance(job_instance, dict):
                    continue
                if job_name != START_LINE and job_name != END_LINE:
                    end_line: int = job_instance.get(END_LINE, -1)
                    end_line_to_job_name_dict[end_line] = job_name

                    steps: list[dict[str, Any]] = [step for step in job_instance.get('steps', []) or [] if step]
                    if not steps:
                        continue

                    for step in steps:
                        if not isinstance(step, dict) or END_LINE not in step:
                            continue
                        end_line_to_job_name_dict[step.get(END_LINE)] = job_name  # type: ignore[index] #
        return end_line_to_job_name_dict



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.checks.enums import BlockType
from checkov.openapi.checks.resource.v2.BaseOpenapiCheckV2 import BaseOpenapiCheckV2


class PathSchemeDefineHTTP(BaseOpenapiCheckV2):
    def __init__(self) -> None:
        # https://learning.postman.com/docs/api-governance/api-definition/openapi2/#schemes-of-the-operation-have-http-scheme-defined
        id = "CKV_OPENAPI_7"
        name = "Ensure that the path scheme does not support unencrypted HTTP connection where all transmissions " \
               "are open to interception- version 2.0 files"
        categories = [CheckCategories.API_SECURITY]
        supported_resources = ['security']
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type=BlockType.DOCUMENT)

    def scan_openapi_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:
        paths = conf.get("paths", {})
        if not paths or not isinstance(paths, dict):
            return CheckResult.UNKNOWN, conf

        for path, http_method in paths.items():
            if self.is_start_end_line(path) or not http_method or not isinstance(http_method, dict):
                continue
            for op_name, op_val in http_method.items():
                if self.is_start_end_line(op_name):
                    continue
                if not isinstance(op_val, dict):
                    continue
                schemes = op_val.get('schemes')
                if schemes and 'http' in schemes:
                    return CheckResult.FAILED, conf
            # If the schemes is not included, the default scheme to be used is the one used to access the Swagger
            # definition itself, in which case the current check is not relevant.

        return CheckResult.PASSED, conf


check = PathSchemeDefineHTTP()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.data.base_check import BaseDataCheck


ADMIN_POLICY_NAME = "AdministratorAccess"
ADMIN_POLICY_ARN = f"arn:aws:iam::aws:policy/{ADMIN_POLICY_NAME}"


class IAMManagedAdminPolicy(BaseDataCheck):
    def __init__(self):
        # This is the full description of your check
        description = "Disallow policies from using the AWS AdministratorAccess policy"

        # This is the Unique ID for your check
        id = "CKV_AWS_275"

        # These are the terraform objects supported by this check (ex: aws_iam_policy_document)
        supported_data = ("aws_iam_policy",)

        # Valid CheckCategories checkov/common/models/enums.py
        categories = (CheckCategories.IAM,)
        super().__init__(name=description, id=id, categories=categories, supported_data=supported_data)

    def scan_data_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "name" in conf.keys():
            if conf.get("name")[0] == ADMIN_POLICY_NAME:
                return CheckResult.FAILED

        if "arn" in conf.keys():
            if conf.get("arn")[0] == ADMIN_POLICY_ARN:
                return CheckResult.FAILED

        return CheckResult.PASSED


check = IAMManagedAdminPolicy()



from typing import Dict, Any

from checkov.arm.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class AzureScaleSetPassword(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Azure linux scale set does not use basic authentication(Use SSH Key Instead)"
        id = "CKV_AZURE_49"
        supported_resources = ("Microsoft.Compute/virtualMachineScaleSets",)
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if isinstance(properties, dict):
            vm_profile = properties.get("virtualMachineProfile")
            if isinstance(vm_profile, dict):
                storage_profile = vm_profile.get("storageProfile")
                if isinstance(storage_profile, dict):
                    image_reference = storage_profile.get("imageReference")
                    if isinstance(image_reference, dict):
                        publisher = image_reference.get("publisher")
                        if publisher and "windows" in publisher.lower():
                            # This check is not relevant to Windows systems
                            return CheckResult.UNKNOWN

        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "properties/virtualMachineProfile/osProfile/linuxConfiguration/disablePasswordAuthentication"

    def get_expected_value(self) -> Any:
        return True


check = AzureScaleSetPassword()



from checkov.serverless.checks.function.base_function_check import BaseFunctionCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.serverless.parsers.parser import IAM_ROLE_STATEMENTS_TOKEN


class AdminPolicyDocument(BaseFunctionCheck):
    def __init__(self) -> None:
        name = "Ensure IAM policies that allow full \"*-*\" administrative privileges are not created"
        id = "CKV_AWS_1"
        supported_entities = ('serverless_aws',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_entities)

    def scan_function_conf(self, conf):
        """
        validates iam policy document
        https://learn.hashicorp.com/terraform/aws/iam-policy
        :param conf: aws_kms_key configuration
        :return: <CheckResult>
        """
        statements = conf.get(IAM_ROLE_STATEMENTS_TOKEN)
        if statements and isinstance(statements, list):
            for statement in statements:
                if 'Action' in statement and statement.get('Effect') == 'Allow' and '*' in statement['Action'] \
                        and '*' in statement['Resource']:
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = AdminPolicyDocument()



from __future__ import annotations
import itertools
import logging

from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.record import SCA_PACKAGE_SCAN_CHECK_NAME, Record
from license_expression import get_spdx_licensing

from io import StringIO

from spdx.creationinfo import Tool, Organization
from spdx.document import Document
from spdx.license import License
from spdx.package import Package
from spdx.writers.tagvalue import write_document

from checkov.common.output.cyclonedx_consts import SCA_CHECKTYPES
from checkov.common.output.report import Report

DOCUMENT_NAME = "checkov-sbom"
SPDXREF = "SPDXRef-"


class SPDX:
    def __init__(self, repo_id: str | None, reports: list[Report]):
        self.repo_id = f"{repo_id}/" if repo_id else ""
        self.reports = reports

        self.document = self.create_document()

    def create_document(self) -> Document:
        document = Document(
            version="SPDX2.3",
            data_license=License.from_identifier(identifier="CC0-1.0"),
            name=DOCUMENT_NAME,
            spdx_id="SPDXRef-DOCUMENT",
            namespace=f"{self.repo_id}{DOCUMENT_NAME}",
        )
        document.creation_info.set_created_now()
        document.creation_info.add_creator(Tool(name="checkov"))
        document.creation_info.add_creator(Organization(name="bridgecrew"))

        return document

    def get_tag_value_output(self) -> str:
        output = StringIO()

        self.add_packages_to_doc()
        write_document(document=self.document, out=output, validate=True)  # later set to True

        return output.getvalue()

    def validate_licenses(self, package: Package, license_: str) -> None:
        if license_ and license_ not in ['Unknown license', 'NOT_FOUND', 'Unknown']:
            split_licenses = license_.split(",")
            licenses = []

            for lic in split_licenses:
                lic = lic.strip('"')
                try:
                    is_spdx_license = License(get_spdx_licensing().parse(lic), lic)
                    licenses.append(is_spdx_license)
                except Exception as e:
                    logging.info(f"error occured when trying to parse the license:{split_licenses} due to error {e}")
            package.licenses_from_files = licenses

    def create_package(self, check: Record | ExtraResource) -> Package:
        package_data = check.vulnerability_details
        if not package_data:
            # this shouldn't happen
            logging.error(f"Check {check.resource} doesn't have 'vulnerability_details' set")
            return Package(name="unknown")

        package_name = package_data.get('package_name')
        package = Package(
            name=package_name,
            spdx_id=f"{SPDXREF}{package_name}",
            version=package_data['package_version'],
            download_location='N/A',
            file_name=check.file_path
        )
        license_ = package_data.get('licenses', "")
        self.validate_licenses(package=package, license_=license_)

        return package

    def add_packages_to_doc(self) -> None:
        packages_set = set()
        for report in self.reports:
            for check in itertools.chain(report.passed_checks, report.skipped_checks):
                if report.check_type in SCA_CHECKTYPES and check.check_name != SCA_PACKAGE_SCAN_CHECK_NAME:
                    continue
                package = self.create_package(check)
                if package not in packages_set:
                    packages_set.add(package)

            for check in report.failed_checks:
                if report.check_type in SCA_CHECKTYPES and check.check_name != SCA_PACKAGE_SCAN_CHECK_NAME:
                    continue
                package = self.create_package(check)
                if package not in packages_set:
                    packages_set.add(package)

            for resource in sorted(report.extra_resources):
                package = self.create_package(resource)
                if package not in packages_set:
                    packages_set.add(package)

        if packages_set:
            self.document.packages = list(packages_set)



from __future__ import annotations

import asyncio
import json
import logging
import os
import time
from collections.abc import Iterable, Sequence, Collection
from pathlib import Path
from typing import Dict, Any, List

import requests

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.platform_key import bridgecrew_dir
from checkov.common.bridgecrew.vulnerability_scanning.image_scanner import image_scanner, TWISTCLI_FILE_NAME
from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import \
    docker_image_scanning_integration
from checkov.common.util.file_utils import compress_file_gzip_base64, decompress_file_gzip_base64
from checkov.common.util.http_utils import request_wrapper

from checkov.common.util.tqdm_utils import ProgressBar

SLEEP_DURATION = 2
MAX_SLEEP_DURATION = 60


class Scanner:
    def __init__(self, pbar: ProgressBar | None = None, root_folder: str | Path | None = None) -> None:
        self._base_url = bc_integration.api_url
        if pbar:
            self.pbar = pbar
        else:
            self.pbar = ProgressBar('')
            self.pbar.turn_off_progress_bar()
        self.root_folder = root_folder

    @staticmethod
    def should_rescan_for_result(scan_result: dict[str, Any] | None) -> bool:
        return scan_result is None or scan_result.get("packages") is None

    def scan(self, input_paths: Collection[Path]) -> Sequence[dict[str, Any]] | None:
        self.pbar.initiate(len(input_paths))
        scan_results = asyncio.run(
            self.run_scan_multi(input_paths=input_paths)
        )
        self.pbar.close()
        return scan_results

    async def run_scan_multi(
            self,
            input_paths: "Iterable[Path]",
    ) -> "Sequence[Dict[str, Any]] | None":

        if os.getenv("PYCHARM_HOSTED") == "1":
            # PYCHARM_HOSTED env variable equals 1 when running via Pycharm.
            # it avoids us from crashing, which happens when using multiprocessing via Pycharm's debug-mode
            logging.warning("Running the scans in sequence for avoiding crashing when running via Pycharm")
            scan_results: list[dict[str, Any] | None] = []
            for input_path in input_paths:
                scan_results.append(await self.run_scan(input_path))
        else:
            scan_results = await asyncio.gather(*[self.run_scan(i) for i in input_paths])

        if any(self.should_rescan_for_result(scan_result) for scan_result in scan_results):
            status: bool = image_scanner.setup_twistcli()

            if not status:
                return None
            if os.getenv("PYCHARM_HOSTED") == "1":
                # PYCHARM_HOSTED env variable equals 1 when running via Pycharm.
                # it avoids us from crashing, which happens when using multiprocessing via Pycharm's debug-mode
                logging.warning("Running the scans in sequence for avoiding crashing when running via Pycharm")
                scan_results = [
                    await self.execute_twistcli_scan(input_path) if self.should_rescan_for_result(scan_results[idx])
                    else scan_results[idx] for idx, input_path in enumerate(input_paths)
                ]
            else:
                input_paths_as_list: List[Path] = list(input_paths)  # create a list from a set ("Iterable")
                indices_to_fix: List[int] = [
                    idx
                    for idx in range(len(input_paths_as_list))
                    if self.should_rescan_for_result(scan_results[idx])
                ]
                new_scan_results = await asyncio.gather(*[
                    self.execute_twistcli_scan(input_paths_as_list[idx]) for idx in indices_to_fix
                ])
                for idx in indices_to_fix:
                    res = new_scan_results.pop(0)
                    res['repository'] = str(input_paths_as_list[idx])
                    scan_results[idx] = res

        # checking whether there are None results that indicates for error. id there is any, we return None
        scan_results_without_nones: list[dict[str, Any]] = []
        for result in scan_results:
            if result is None:
                return None
            else:
                scan_results_without_nones.append(result)

        return scan_results_without_nones

    async def run_scan(self, input_path: Path) -> dict[str, Any] | None:
        try:
            self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(input_path, self.root_folder)})
            logging.info(f"Start to scan package file {input_path}")

            request_body = {
                "compressedFileBody": compress_file_gzip_base64(str(input_path)),
                "compressionMethod": "gzip",
                "fileName": input_path.name
            }

            response = request_wrapper(
                "POST", f"{self._base_url}/api/v1/vulnerabilities/scan",
                headers=bc_integration.get_default_headers("POST"),
                json=request_body,
                should_call_raise_for_status=True
            )

            response_json = response.json()

            if response_json["status"] == "already_exist":
                logging.info(f"result for {input_path} exists in the cache")
                return self.parse_api_result(input_path, response_json["outputData"])

            return self.run_scan_busy_wait(input_path, response_json['id'])
        except Exception:
            logging.debug(
                "[sca_package] - Unexpected failure happened during package scanning.\n"
                "the scanning is terminating. details are below.\n"
                "please try again. if it is repeated, please report.", exc_info=True)
            return None

    def run_scan_busy_wait(self, input_path: Path, scan_id: str) -> dict[str, Any]:
        current_state = "Empty"
        desired_state = "Result"
        total_sleeping_time = 0
        response = requests.Response()

        while current_state != desired_state:
            response = request_wrapper(
                "GET", f"{self._base_url}/api/v1/vulnerabilities/scan-results/{scan_id}",
                headers=bc_integration.get_default_headers("GET")
            )
            response_json = response.json()
            current_state = response_json["outputType"]

            if current_state == "Error":
                logging.error(response_json["outputData"])
                return {}

            if total_sleeping_time > MAX_SLEEP_DURATION:
                logging.info(f"Timeout, slept for {total_sleeping_time}")
                return {}

            time.sleep(SLEEP_DURATION)
            total_sleeping_time += SLEEP_DURATION

        return self.parse_api_result(input_path, response.json()["outputData"])

    def parse_api_result(self, origin_file_path: Path, response: str) -> dict[str, Any]:
        raw_result: dict[str, Any] = json.loads(decompress_file_gzip_base64(response))
        raw_result['repository'] = str(origin_file_path)
        self.pbar.update()
        return raw_result

    async def execute_twistcli_scan(
            self,
            input_path: Path,
    ) -> Dict[str, Any]:
        output_path = Path(f'results-{input_path.name}.json')

        command = f"{Path(bridgecrew_dir) / TWISTCLI_FILE_NAME} coderepo scan --address {docker_image_scanning_integration.get_proxy_address()} --token {docker_image_scanning_integration.get_bc_api_key()} --details --output-file \"{output_path}\" {input_path}"
        process = await asyncio.create_subprocess_shell(
            command, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
        )

        stdout, stderr = await process.communicate()

        # log output for debugging
        logging.debug(stdout.decode())

        exit_code = await process.wait()

        if exit_code:
            logging.error(stderr.decode())
            return {}

        # read the report file
        scan_result: Dict[str, Any] = json.loads(output_path.read_text())
        output_path.unlink()
        return scan_result



from __future__ import annotations

from checkov.github.base_github_branch_security import BranchSecurity


class GithubBranchRequireConversationResolution(BranchSecurity):
    def __init__(self) -> None:
        name = "Ensure GitHub branch protection requires conversation resolution"
        id = "CKV_GITHUB_16"
        super().__init__(name=name, id=id)

    def get_evaluated_keys(self) -> list[str]:
        return ["required_conversation_resolution/enabled"]


check = GithubBranchRequireConversationResolution()



from __future__ import annotations

import os
from typing import List, Dict, Any, Tuple

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.runners.base_runner import strtobool
from checkov.common.util.parser_utils import get_tf_definition_key_from_module_dependency
from checkov.terraform.modules.module_objects import TFDefinitionKey
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import TerraformBlock


def convert_graph_vertices_to_tf_definitions(
    vertices: List[TerraformBlock], root_folder: str
) -> Tuple[Dict[str | TFDefinitionKey, Dict[str, Any]], Dict[str, Dict[str, Any]]]:
    use_new_tf_parser = strtobool(os.getenv('CHECKOV_NEW_TF_PARSER', 'True'))
    tf_definitions: Dict[str | TFDefinitionKey, Dict[str, Any]] = {}
    breadcrumbs: Dict[str, Dict[str, Any]] = {}
    for vertex in vertices:
        block_path = vertex.path
        if not os.path.isfile(block_path):
            print(f"tried to convert vertex to tf_definitions but its path doesnt exist: {vertex}")
            continue
        block_type = vertex.block_type
        if block_type == BlockType.TF_VARIABLE:
            continue

        tf_path: str | TFDefinitionKey = TFDefinitionKey(file_path=block_path) if use_new_tf_parser else block_path
        if vertex.module_dependency or hasattr(vertex, "source_module_object") and vertex.source_module_object:
            if use_new_tf_parser:
                tf_path = TFDefinitionKey(file_path=block_path, tf_source_modules=vertex.source_module_object)
            else:
                tf_path = get_tf_definition_key_from_module_dependency(block_path, vertex.module_dependency, vertex.module_dependency_num)
        tf_definitions.setdefault(tf_path, {}).setdefault(block_type, []).append(vertex.config)
        relative_block_path = f"/{os.path.relpath(block_path, root_folder)}"
        add_breadcrumbs(vertex, breadcrumbs, relative_block_path)
    return tf_definitions, breadcrumbs


def add_breadcrumbs(vertex: TerraformBlock, breadcrumbs: Dict[str, Dict[str, Any]], relative_block_path: str) -> None:
    vertex_breadcrumbs = vertex.breadcrumbs
    if vertex_breadcrumbs:
        vertex_key = vertex.name if not strtobool(os.getenv('CHECKOV_ENABLE_NESTED_MODULES', 'True')) else vertex.attributes.get(CustomAttributes.TF_RESOURCE_ADDRESS, vertex.name)
        breadcrumbs.setdefault(relative_block_path, {})[vertex_key] = vertex_breadcrumbs



from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceRemoteDebuggingNotEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that remote debugging is not enabled for app services"
        id = "CKV_AZURE_72"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "site_config/[0]/remote_debugging_enabled"

    def get_expected_value(self) -> Any:
        return False


check = AppServiceRemoteDebuggingNotEnabled()




from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SagemakerNotebookRoot(BaseResourceValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-2(1), NIST.800-53.r5 AC-3(15), NIST.800-53.r5 AC-3(7), NIST.800-53.r5 AC-6, NIST.800-53.r5
        AC-6(10), NIST.800-53.r5 AC-6(2)
        """
        name = "Ensure SageMaker Users should not have root access to SageMaker notebook instances"
        id = "CKV_AWS_307"
        supported_resources = ['aws_sagemaker_notebook_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return 'root_access'

    def get_expected_value(self):
        return "Disabled"


check = SagemakerNotebookRoot()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class RDSInstancePerformanceInsights(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that RDS instances have performance insights enabled"
        id = "CKV_AWS_353"
        supported_resources = ('aws_rds_cluster_instance', 'aws_db_instance')
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'performance_insights_enabled'


check = RDSInstancePerformanceInsights()



from checkov.common.models.consts import ANY_VALUE
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class SpannerDatabaseEncryptedWithCMK(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Spanner Database is encrypted with Customer Supplied Encryption Keys (CSEK)"
        id = "CKV_GCP_93"
        supported_resources = ['google_spanner_database']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'encryption_config/[0]/kms_key_name'

    def get_expected_value(self):
        return ANY_VALUE


check = SpannerDatabaseEncryptedWithCMK()



from __future__ import annotations

import logging
import re
from pathlib import Path
from typing import Any

from checkov.ansible.graph_builder.graph_components.resource_types import ResourceType
from checkov.common.parsers.yaml.parser import parse
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.file_utils import read_file_with_any_encoding
from checkov.common.util.suppression import collect_suppressions_for_context

TASK_NAME_PATTERN = re.compile(r"^\s*-\s+name:\s+", re.MULTILINE)

# https://docs.ansible.com/ansible/latest/reference_appendices/playbooks_keywords.html#task
TASK_RESERVED_KEYWORDS = {
    "action",
    "any_errors_fatal",
    "args",
    "async",
    "become",
    "become_exe",
    "become_flags",
    "become_method",
    "become_user",
    "changed_when",
    "check_mode",
    "collections",
    "connection",
    "debugger",
    "delay",
    "delegate_facts",
    "delegate_to",
    "diff",
    "environment",
    "failed_when",
    "ignore_errors",
    "ignore_unreachable",
    "local_action",
    "loop",
    "loop_control",
    "module_defaults",
    "name",
    "no_log",
    "notify",
    "poll",
    "port",
    "register",
    "remote_user",
    "retries",
    "run_once",
    "tags",
    "throttle",
    "timeout",
    "until",
    "vars",
    "when",
}

logger = logging.getLogger(__name__)


def get_scannable_file_paths(root_folder: str | Path) -> set[Path]:
    """Finds yaml files"""

    file_paths: set[Path] = set()

    if root_folder:
        root_path = root_folder if isinstance(root_folder, Path) else Path(root_folder)
        file_paths = {file_path for file_path in root_path.rglob("*.[y][am]*[l]") if file_path.is_file()}

    return file_paths


def get_relevant_file_content(file_path: str | Path) -> str | None:
    if not str(file_path).endswith((".yaml", ".yml")):
        return None

    content = read_file_with_any_encoding(file_path=file_path)
    match_task_name = re.search(TASK_NAME_PATTERN, content)
    if match_task_name:
        # there are more files, which belong to an ansible playbook,
        # but we are currently only interested in 'tasks'
        return content

    return None


def parse_file(
    f: str | Path, file_content: str | None = None
) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
    file_content = get_relevant_file_content(file_path=f)
    if file_content:
        content = parse(filename=str(f), file_content=file_content)
        return content

    return None


def generate_task_name(task: dict[str, Any], prefix: str = "") -> str | None:
    # grab the task name at the beginning before trying to find the actual module name
    task_name = task.get("name") or "unknown"

    for name in task:
        if name in TASK_RESERVED_KEYWORDS:
            continue

        if prefix:
            # if the task is found in a block, then prefix the module name with 'block'
            name = f"{prefix}{name}"

        return f"{ResourceType.TASKS}.{name}.{task_name}"

    return None


def build_definitions_context(
    definitions: dict[str, dict[str, Any] | list[dict[str, Any]]],
    definitions_raw: dict[str, list[tuple[int, str]]],
) -> dict[str, dict[str, Any]]:
    definitions_context: dict[str, dict[str, Any]] = {}

    for file_path, definition in definitions.items():
        file_path_context: dict[str, Any] = {}
        definition_raw = definitions_raw[file_path]

        if not isinstance(definition, list):
            logger.info(f"File {file_path} has the wrong type {type(definition)}")
            continue

        for code_block in definition:
            if ResourceType.TASKS in code_block:
                for task in code_block[ResourceType.TASKS]:
                    _process_blocks(definition_raw=definition_raw, file_path_context=file_path_context, task=task)
            else:
                _process_blocks(definition_raw=definition_raw, file_path_context=file_path_context, task=code_block)

        definitions_context[file_path] = file_path_context

    return definitions_context


def _process_blocks(
    definition_raw: list[tuple[int, str]],
    file_path_context: dict[str, Any],
    task: Any,
    prefix: str = "",
) -> None:
    """Checks for possible block usage"""

    if not task or not isinstance(task, dict):
        return

    if ResourceType.BLOCK in task and isinstance(task[ResourceType.BLOCK], list):
        prefix += f"{ResourceType.BLOCK}."  # with each nested level an extra block prefix is added
        block_name = f"{prefix}.{task.get('name') or 'unknown'}"
        resource_context = _create_resource_context(definition_raw=definition_raw, resource=task)
        file_path_context[block_name] = resource_context

        for block_task in task[ResourceType.BLOCK]:
            _process_blocks(
                definition_raw=definition_raw, file_path_context=file_path_context, task=block_task, prefix=prefix
            )
    else:
        resource_context = _create_resource_context(definition_raw=definition_raw, resource=task)
        task_name = generate_task_name(task=task, prefix=prefix)
        if task_name:
            file_path_context[task_name] = resource_context


def _create_resource_context(definition_raw: list[tuple[int, str]], resource: dict[str, Any]) -> dict[str, Any]:
    """Creates the resource context block"""

    start_line = resource[START_LINE]
    end_line = resource[END_LINE]
    code_lines = definition_raw[start_line - 1 : end_line - 1]  # lines start with index 0
    skipped_checks = collect_suppressions_for_context(code_lines=code_lines)

    return {
        "start_line": start_line,
        "end_line": end_line - 1,
        "code_lines": code_lines,
        "skipped_checks": skipped_checks,
    }



from typing import Dict, Any

from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list

# https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2019-06-01-preview/servers
# https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2017-03-01-preview/servers/databases/auditingsettings


class SQLServerAuditingRetention90Days(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that 'Auditing' Retention is 'greater than 90 days' for SQL servers"
        id = "CKV_AZURE_24"
        supported_resources = ("Microsoft.Sql/servers",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_keys = ["resources"]
        resources = conf.get("resources") or []
        for idx, resource in enumerate(force_list(resources)):
            self.evaluated_keys = [
                f"resources/[{idx}]/type",
                f"resources/[{idx}]/properties/state",
                f"resources/[{idx}]/properties/retentionDays",
            ]
            if resource.get("type") in (
                "Microsoft.Sql/servers/databases/auditingSettings",
                'Microsoft.Sql/servers/auditingSettings',
                "auditingSettings",
            ):
                return self.check_resource(resource)
            elif resource.get("type") in (
                "databases"
            ):
                sub_resources = resource.get("resources") or []
                for sr in sub_resources:
                    if sr.get("type") == "Microsoft.Sql/servers/databases/auditingPolicies":
                        return self.check_resource(sr)

        return CheckResult.FAILED

    @staticmethod
    def check_resource(resource: Dict[str, Any]) -> CheckResult:
        properties = resource.get("properties")
        if isinstance(properties, dict):
            state = properties.get("state")
            if isinstance(state, str) and state.lower() == "enabled":
                retention = properties.get("retentionDays")
                if isinstance(retention, int) and retention >= 90:
                    return CheckResult.PASSED
        return CheckResult.FAILED


check = SQLServerAuditingRetention90Days()
